# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/address_add.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AddressAddDialog(object):
    def setupUi(self, AddressAddDialog):
        AddressAddDialog.setObjectName("AddressAddDialog")
        AddressAddDialog.resize(629, 394)
        self.verticalLayout = QtWidgets.QVBoxLayout(AddressAddDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.imageLabel = QtWidgets.QLabel(AddressAddDialog)
        self.imageLabel.setMaximumSize(QtCore.QSize(200, 200))
        self.imageLabel.setText("")
        self.imageLabel.setPixmap(QtGui.QPixmap(":/images/safe_girl"))
        self.imageLabel.setScaledContents(True)
        self.imageLabel.setObjectName("imageLabel")
        self.verticalLayout.addWidget(self.imageLabel, 0, QtCore.Qt.AlignHCenter)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.balanceLabel = QtWidgets.QLabel(AddressAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.balanceLabel.setFont(font)
        self.balanceLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.balanceLabel.setObjectName("balanceLabel")
        self.gridLayout.addWidget(self.balanceLabel, 2, 0, 1, 1)
        self.balanceValueLabel = QtWidgets.QLabel(AddressAddDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.balanceValueLabel.sizePolicy().hasHeightForWidth())
        self.balanceValueLabel.setSizePolicy(sizePolicy)
        self.balanceValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.balanceValueLabel.setObjectName("balanceValueLabel")
        self.gridLayout.addWidget(self.balanceValueLabel, 2, 1, 1, 1)
        self.addressLineEdit = QtWidgets.QLineEdit(AddressAddDialog)
        self.addressLineEdit.setReadOnly(False)
        self.addressLineEdit.setObjectName("addressLineEdit")
        self.gridLayout.addWidget(self.addressLineEdit, 0, 1, 1, 1)
        self.loaderIconLabel = QtWidgets.QLabel(AddressAddDialog)
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setText("")
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.gridLayout.addWidget(self.loaderIconLabel, 0, 2, 1, 1)
        self.keyTypeLabel = QtWidgets.QLabel(AddressAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.keyTypeLabel.setFont(font)
        self.keyTypeLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.keyTypeLabel.setObjectName("keyTypeLabel")
        self.gridLayout.addWidget(self.keyTypeLabel, 1, 0, 1, 1)
        self.keyTypeValueLabel = QtWidgets.QLabel(AddressAddDialog)
        self.keyTypeValueLabel.setObjectName("keyTypeValueLabel")
        self.gridLayout.addWidget(self.keyTypeValueLabel, 1, 1, 1, 1)
        self.addressLabel = QtWidgets.QLabel(AddressAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.addressLabel.setObjectName("addressLabel")
        self.gridLayout.addWidget(self.addressLabel, 0, 0, 1, 1)
        self.nameLabel = QtWidgets.QLabel(AddressAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nameLabel.setFont(font)
        self.nameLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nameLabel.setObjectName("nameLabel")
        self.gridLayout.addWidget(self.nameLabel, 3, 0, 1, 1)
        self.nameLineEdit = QtWidgets.QLineEdit(AddressAddDialog)
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.gridLayout.addWidget(self.nameLineEdit, 3, 1, 1, 1)
        self.identityIconLabel = QtWidgets.QLabel(AddressAddDialog)
        self.identityIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.identityIconLabel.setText("")
        self.identityIconLabel.setObjectName("identityIconLabel")
        self.gridLayout.addWidget(self.identityIconLabel, 2, 2, 1, 1)
        self.identityNameAndIndexValuelabel = QtWidgets.QLabel(AddressAddDialog)
        self.identityNameAndIndexValuelabel.setStyleSheet("color: blue")
        self.identityNameAndIndexValuelabel.setText("")
        self.identityNameAndIndexValuelabel.setObjectName("identityNameAndIndexValuelabel")
        self.gridLayout.addWidget(self.identityNameAndIndexValuelabel, 3, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.errorLabel = QtWidgets.QLabel(AddressAddDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorLabel.sizePolicy().hasHeightForWidth())
        self.errorLabel.setSizePolicy(sizePolicy)
        self.errorLabel.setStyleSheet("color: rgb(255, 0, 0)")
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel, 0, QtCore.Qt.AlignHCenter)
        self.buttonBox = QtWidgets.QDialogButtonBox(AddressAddDialog)
        self.buttonBox.setEnabled(True)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(AddressAddDialog)
        self.buttonBox.accepted.connect(AddressAddDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(AddressAddDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(AddressAddDialog)

    def retranslateUi(self, AddressAddDialog):
        #  _translate = QtCore.QCoreApplication.translate
        AddressAddDialog.setWindowTitle(_("Add an address"))
        self.balanceLabel.setText(_("Balance"))
        self.balanceValueLabel.setText(_("?"))
        self.keyTypeLabel.setText(_("Key type"))
        self.keyTypeValueLabel.setText(_("?"))
        self.addressLabel.setText(_("Address"))
        self.nameLabel.setText(_("Add a name"))
import tikka.slots.pyqt.resources.icons.index_rc
