# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountWidget(object):
    def setupUi(self, AccountWidget):
        AccountWidget.setObjectName("AccountWidget")
        AccountWidget.resize(841, 401)
        AccountWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(AccountWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.QRCodeAddressLabel = QtWidgets.QLabel(AccountWidget)
        self.QRCodeAddressLabel.setMaximumSize(QtCore.QSize(100, 100))
        self.QRCodeAddressLabel.setBaseSize(QtCore.QSize(200, 0))
        self.QRCodeAddressLabel.setText("")
        self.QRCodeAddressLabel.setPixmap(QtGui.QPixmap(":/images/safe"))
        self.QRCodeAddressLabel.setScaledContents(True)
        self.QRCodeAddressLabel.setObjectName("QRCodeAddressLabel")
        self.horizontalLayout.addWidget(self.QRCodeAddressLabel)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.balanceValueLabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.balanceValueLabel.sizePolicy().hasHeightForWidth())
        self.balanceValueLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(40)
        self.balanceValueLabel.setFont(font)
        self.balanceValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 25%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.balanceValueLabel.setObjectName("balanceValueLabel")
        self.verticalLayout_3.addWidget(self.balanceValueLabel)
        self.balanceReservedValueLabel = QtWidgets.QLabel(AccountWidget)
        self.balanceReservedValueLabel.setStyleSheet("color: orange")
        self.balanceReservedValueLabel.setText("")
        self.balanceReservedValueLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.balanceReservedValueLabel.setObjectName("balanceReservedValueLabel")
        self.verticalLayout_3.addWidget(self.balanceReservedValueLabel)
        self.horizontalLayout.addLayout(self.verticalLayout_3)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.refreshButton = QtWidgets.QPushButton(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.verticalLayout.addWidget(self.refreshButton, 0, QtCore.Qt.AlignLeft)
        self.transferToButton = QtWidgets.QPushButton(AccountWidget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/money-recive-svgrepo-com.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.transferToButton.setIcon(icon1)
        self.transferToButton.setIconSize(QtCore.QSize(32, 32))
        self.transferToButton.setObjectName("transferToButton")
        self.verticalLayout.addWidget(self.transferToButton)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setContentsMargins(6, -1, 0, -1)
        self.formLayout_2.setObjectName("formLayout_2")
        self.lockStatusIcon = QtWidgets.QLabel(AccountWidget)
        self.lockStatusIcon.setMaximumSize(QtCore.QSize(16, 16))
        self.lockStatusIcon.setText("")
        self.lockStatusIcon.setPixmap(QtGui.QPixmap(":/images/locked_cartoon_400x400.png"))
        self.lockStatusIcon.setScaledContents(True)
        self.lockStatusIcon.setObjectName("lockStatusIcon")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lockStatusIcon)
        self.addressValueLabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addressValueLabel.sizePolicy().hasHeightForWidth())
        self.addressValueLabel.setSizePolicy(sizePolicy)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.addressValueLabel)
        self.derivationValueLabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.derivationValueLabel.sizePolicy().hasHeightForWidth())
        self.derivationValueLabel.setSizePolicy(sizePolicy)
        self.derivationValueLabel.setText("")
        self.derivationValueLabel.setObjectName("derivationValueLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.derivationValueLabel)
        self.derivationIconLabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.derivationIconLabel.sizePolicy().hasHeightForWidth())
        self.derivationIconLabel.setSizePolicy(sizePolicy)
        self.derivationIconLabel.setMaximumSize(QtCore.QSize(16, 20))
        self.derivationIconLabel.setText("")
        self.derivationIconLabel.setPixmap(QtGui.QPixmap(":/icons/wot_icon"))
        self.derivationIconLabel.setScaledContents(True)
        self.derivationIconLabel.setObjectName("derivationIconLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.derivationIconLabel)
        self.v1AddressValueLabel = QtWidgets.QLabel(AccountWidget)
        font = QtGui.QFont()
        font.setItalic(True)
        self.v1AddressValueLabel.setFont(font)
        self.v1AddressValueLabel.setText("")
        self.v1AddressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.v1AddressValueLabel.setObjectName("v1AddressValueLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.v1AddressValueLabel)
        self.horizontalLayout_3.addLayout(self.formLayout_2)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.identityStatusIconLabel = QtWidgets.QLabel(AccountWidget)
        self.identityStatusIconLabel.setMaximumSize(QtCore.QSize(16, 20))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.identityStatusIconLabel.setFont(font)
        self.identityStatusIconLabel.setText("")
        self.identityStatusIconLabel.setPixmap(QtGui.QPixmap(":/icons/member_icon"))
        self.identityStatusIconLabel.setScaledContents(True)
        self.identityStatusIconLabel.setObjectName("identityStatusIconLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.identityStatusIconLabel)
        self.identityValueLabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityValueLabel.sizePolicy().hasHeightForWidth())
        self.identityValueLabel.setSizePolicy(sizePolicy)
        self.identityValueLabel.setObjectName("identityValueLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.identityValueLabel)
        self.smithLabel = QtWidgets.QLabel(AccountWidget)
        self.smithLabel.setMaximumSize(QtCore.QSize(16, 20))
        self.smithLabel.setText("")
        self.smithLabel.setPixmap(QtGui.QPixmap(":/icons/smith"))
        self.smithLabel.setScaledContents(True)
        self.smithLabel.setObjectName("smithLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.smithLabel)
        self.smithValuelabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.smithValuelabel.sizePolicy().hasHeightForWidth())
        self.smithValuelabel.setSizePolicy(sizePolicy)
        self.smithValuelabel.setObjectName("smithValuelabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.smithValuelabel)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSpacing(0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.identityNameValueLabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityNameValueLabel.sizePolicy().hasHeightForWidth())
        self.identityNameValueLabel.setSizePolicy(sizePolicy)
        self.identityNameValueLabel.setStyleSheet("color: blue")
        self.identityNameValueLabel.setObjectName("identityNameValueLabel")
        self.horizontalLayout_4.addWidget(self.identityNameValueLabel)
        self.accountNameValueLabel = QtWidgets.QLabel(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountNameValueLabel.sizePolicy().hasHeightForWidth())
        self.accountNameValueLabel.setSizePolicy(sizePolicy)
        self.accountNameValueLabel.setObjectName("accountNameValueLabel")
        self.horizontalLayout_4.addWidget(self.accountNameValueLabel)
        self.formLayout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_4)
        self.horizontalLayout_3.addLayout(self.formLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.errorLabel = QtWidgets.QLabel(AccountWidget)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout_2.addWidget(self.errorLabel)
        self.groupBox = QtWidgets.QGroupBox(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.transfersCountValueLabel = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.transfersCountValueLabel.sizePolicy().hasHeightForWidth())
        self.transfersCountValueLabel.setSizePolicy(sizePolicy)
        self.transfersCountValueLabel.setText("")
        self.transfersCountValueLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.transfersCountValueLabel.setObjectName("transfersCountValueLabel")
        self.horizontalLayout_2.addWidget(self.transfersCountValueLabel)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.transfersListView = QtWidgets.QListView(self.groupBox)
        self.transfersListView.setMouseTracking(True)
        self.transfersListView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.transfersListView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.transfersListView.setAlternatingRowColors(True)
        self.transfersListView.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.transfersListView.setObjectName("transfersListView")
        self.verticalLayout_4.addWidget(self.transfersListView)
        self.verticalLayout_2.addWidget(self.groupBox)

        self.retranslateUi(AccountWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountWidget)

    def retranslateUi(self, AccountWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountWidget.setWindowTitle(_("Account"))
        self.balanceValueLabel.setText(_("?"))
        self.refreshButton.setText(_("Refresh"))
        self.transferToButton.setText(_("Transfer to"))
        self.identityValueLabel.setText(_("None"))
        self.smithValuelabel.setText(_("None"))
        self.identityNameValueLabel.setText(_("identity name"))
        self.accountNameValueLabel.setText(_("account name"))
        self.groupBox.setTitle(_("Transfers"))
import tikka.slots.pyqt.resources.icons.index_rc
