# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/technical_committee.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_technicalCommitteeWidget(object):
    def setupUi(self, technicalCommitteeWidget):
        technicalCommitteeWidget.setObjectName("technicalCommitteeWidget")
        technicalCommitteeWidget.resize(811, 653)
        self.verticalLayout = QtWidgets.QVBoxLayout(technicalCommitteeWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.refreshButton = QtWidgets.QPushButton(technicalCommitteeWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/loader"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.verticalLayout.addWidget(self.refreshButton, 0, QtCore.Qt.AlignRight)
        self.membersGroupBox = QtWidgets.QGroupBox(technicalCommitteeWidget)
        self.membersGroupBox.setObjectName("membersGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.membersGroupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.membersListWidget = QtWidgets.QListWidget(self.membersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.membersListWidget.sizePolicy().hasHeightForWidth())
        self.membersListWidget.setSizePolicy(sizePolicy)
        self.membersListWidget.setObjectName("membersListWidget")
        self.verticalLayout_2.addWidget(self.membersListWidget)
        self.verticalLayout.addWidget(self.membersGroupBox)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.accountLabel = QtWidgets.QLabel(technicalCommitteeWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.accountLabel.setFont(font)
        self.accountLabel.setObjectName("accountLabel")
        self.horizontalLayout.addWidget(self.accountLabel)
        self.accountComboBox = QtWidgets.QComboBox(technicalCommitteeWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountComboBox.sizePolicy().hasHeightForWidth())
        self.accountComboBox.setSizePolicy(sizePolicy)
        self.accountComboBox.setObjectName("accountComboBox")
        self.horizontalLayout.addWidget(self.accountComboBox)
        self.loaderIconLabel = QtWidgets.QLabel(technicalCommitteeWidget)
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setPixmap(QtGui.QPixmap(":/icons/loader"))
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.horizontalLayout.addWidget(self.loaderIconLabel)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.proposalsGroupBox = QtWidgets.QGroupBox(technicalCommitteeWidget)
        self.proposalsGroupBox.setObjectName("proposalsGroupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.proposalsGroupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.proposalsListWidget = QtWidgets.QListWidget(self.proposalsGroupBox)
        self.proposalsListWidget.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.proposalsListWidget.setObjectName("proposalsListWidget")
        self.verticalLayout_3.addWidget(self.proposalsListWidget)
        self.verticalLayout.addWidget(self.proposalsGroupBox)
        self.errorLabel = QtWidgets.QLabel(technicalCommitteeWidget)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)

        self.retranslateUi(technicalCommitteeWidget)
        QtCore.QMetaObject.connectSlotsByName(technicalCommitteeWidget)

    def retranslateUi(self, technicalCommitteeWidget):
        #  _translate = QtCore.QCoreApplication.translate
        technicalCommitteeWidget.setWindowTitle(_("Form"))
        self.refreshButton.setText(_("Refresh"))
        self.membersGroupBox.setTitle(_("Members"))
        self.accountLabel.setText(_("Voting account"))
        self.proposalsGroupBox.setTitle(_("Proposals"))
import tikka.slots.pyqt.resources.icons.index_rc
