# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/node_add.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_nodeAddDialog(object):
    def setupUi(self, nodeAddDialog):
        nodeAddDialog.setObjectName("nodeAddDialog")
        nodeAddDialog.resize(731, 241)
        self.verticalLayout = QtWidgets.QVBoxLayout(nodeAddDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.softwareValueLabel = QtWidgets.QLabel(nodeAddDialog)
        self.softwareValueLabel.setText("")
        self.softwareValueLabel.setObjectName("softwareValueLabel")
        self.gridLayout.addWidget(self.softwareValueLabel, 1, 1, 1, 1)
        self.versionValueLabel = QtWidgets.QLabel(nodeAddDialog)
        self.versionValueLabel.setText("")
        self.versionValueLabel.setObjectName("versionValueLabel")
        self.gridLayout.addWidget(self.versionValueLabel, 2, 1, 1, 1)
        self.urlLabel = QtWidgets.QLabel(nodeAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.urlLabel.setFont(font)
        self.urlLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.urlLabel.setObjectName("urlLabel")
        self.gridLayout.addWidget(self.urlLabel, 0, 0, 1, 1)
        self.blockLabel = QtWidgets.QLabel(nodeAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.blockLabel.setFont(font)
        self.blockLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.blockLabel.setObjectName("blockLabel")
        self.gridLayout.addWidget(self.blockLabel, 4, 0, 1, 1)
        self.versionLabel = QtWidgets.QLabel(nodeAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.versionLabel.setFont(font)
        self.versionLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.versionLabel.setObjectName("versionLabel")
        self.gridLayout.addWidget(self.versionLabel, 2, 0, 1, 1)
        self.softwareLabel = QtWidgets.QLabel(nodeAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.softwareLabel.setFont(font)
        self.softwareLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.softwareLabel.setObjectName("softwareLabel")
        self.gridLayout.addWidget(self.softwareLabel, 1, 0, 1, 1)
        self.currencyHashLabel = QtWidgets.QLabel(nodeAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.currencyHashLabel.setFont(font)
        self.currencyHashLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.currencyHashLabel.setObjectName("currencyHashLabel")
        self.gridLayout.addWidget(self.currencyHashLabel, 5, 0, 1, 1)
        self.currencyHashValueLabel = QtWidgets.QLabel(nodeAddDialog)
        self.currencyHashValueLabel.setText("")
        self.currencyHashValueLabel.setObjectName("currencyHashValueLabel")
        self.gridLayout.addWidget(self.currencyHashValueLabel, 5, 1, 1, 1)
        self.peerIDValueLabel = QtWidgets.QLabel(nodeAddDialog)
        self.peerIDValueLabel.setText("")
        self.peerIDValueLabel.setObjectName("peerIDValueLabel")
        self.gridLayout.addWidget(self.peerIDValueLabel, 3, 1, 1, 1)
        self.peerIDLabel = QtWidgets.QLabel(nodeAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.peerIDLabel.setFont(font)
        self.peerIDLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.peerIDLabel.setObjectName("peerIDLabel")
        self.gridLayout.addWidget(self.peerIDLabel, 3, 0, 1, 1)
        self.urlLineEdit = QtWidgets.QLineEdit(nodeAddDialog)
        self.urlLineEdit.setObjectName("urlLineEdit")
        self.gridLayout.addWidget(self.urlLineEdit, 0, 1, 1, 1)
        self.testButton = QtWidgets.QPushButton(nodeAddDialog)
        self.testButton.setObjectName("testButton")
        self.gridLayout.addWidget(self.testButton, 0, 2, 1, 1)
        self.blockValueLabel = QtWidgets.QLabel(nodeAddDialog)
        self.blockValueLabel.setText("")
        self.blockValueLabel.setObjectName("blockValueLabel")
        self.gridLayout.addWidget(self.blockValueLabel, 4, 1, 1, 1)
        self.currencyPrefixLabel = QtWidgets.QLabel(nodeAddDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.currencyPrefixLabel.setFont(font)
        self.currencyPrefixLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.currencyPrefixLabel.setObjectName("currencyPrefixLabel")
        self.gridLayout.addWidget(self.currencyPrefixLabel, 6, 0, 1, 1)
        self.currencyPrefixValueLabel = QtWidgets.QLabel(nodeAddDialog)
        self.currencyPrefixValueLabel.setText("")
        self.currencyPrefixValueLabel.setObjectName("currencyPrefixValueLabel")
        self.gridLayout.addWidget(self.currencyPrefixValueLabel, 6, 1, 1, 1)
        self.loaderIconLabel = QtWidgets.QLabel(nodeAddDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loaderIconLabel.sizePolicy().hasHeightForWidth())
        self.loaderIconLabel.setSizePolicy(sizePolicy)
        self.loaderIconLabel.setMinimumSize(QtCore.QSize(0, 0))
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setText("")
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.gridLayout.addWidget(self.loaderIconLabel, 1, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.errorLabel = QtWidgets.QLabel(nodeAddDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorLabel.sizePolicy().hasHeightForWidth())
        self.errorLabel.setSizePolicy(sizePolicy)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.buttonBox = QtWidgets.QDialogButtonBox(nodeAddDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(nodeAddDialog)
        self.buttonBox.accepted.connect(nodeAddDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(nodeAddDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(nodeAddDialog)
        nodeAddDialog.setTabOrder(self.urlLineEdit, self.testButton)

    def retranslateUi(self, nodeAddDialog):
        #  _translate = QtCore.QCoreApplication.translate
        nodeAddDialog.setWindowTitle(_("Add server"))
        self.urlLabel.setText(_("URL"))
        self.blockLabel.setText(_("Block"))
        self.versionLabel.setText(_("Version"))
        self.softwareLabel.setText(_("Software"))
        self.currencyHashLabel.setText(_("Currency Hash"))
        self.peerIDLabel.setText(_("Peer ID"))
        self.testButton.setText(_("Test"))
        self.currencyPrefixLabel.setText(_("Currency Prefix"))
import tikka.slots.pyqt.resources.icons.index_rc
