# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/connection.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ConnectionWidget(object):
    def setupUi(self, ConnectionWidget):
        ConnectionWidget.setObjectName("ConnectionWidget")
        ConnectionWidget.resize(724, 424)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(ConnectionWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(ConnectionWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.unsafeAPIExposedValueLabel = QtWidgets.QLabel(self.groupBox)
        self.unsafeAPIExposedValueLabel.setText("")
        self.unsafeAPIExposedValueLabel.setObjectName("unsafeAPIExposedValueLabel")
        self.gridLayout.addWidget(self.unsafeAPIExposedValueLabel, 7, 1, 1, 1)
        self.unsafeAPIExposedLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.unsafeAPIExposedLabel.setFont(font)
        self.unsafeAPIExposedLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unsafeAPIExposedLabel.setObjectName("unsafeAPIExposedLabel")
        self.gridLayout.addWidget(self.unsafeAPIExposedLabel, 7, 0, 1, 1)
        self.nodeBlockValueLabel = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nodeBlockValueLabel.sizePolicy().hasHeightForWidth())
        self.nodeBlockValueLabel.setSizePolicy(sizePolicy)
        self.nodeBlockValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);")
        self.nodeBlockValueLabel.setObjectName("nodeBlockValueLabel")
        self.gridLayout.addWidget(self.nodeBlockValueLabel, 5, 1, 1, 1)
        self.nodeBlockLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nodeBlockLabel.setFont(font)
        self.nodeBlockLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nodeBlockLabel.setObjectName("nodeBlockLabel")
        self.gridLayout.addWidget(self.nodeBlockLabel, 5, 0, 1, 1)
        self.versionLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.versionLabel.setFont(font)
        self.versionLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.versionLabel.setObjectName("versionLabel")
        self.gridLayout.addWidget(self.versionLabel, 3, 0, 1, 1)
        self.epochValueLabel = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.epochValueLabel.sizePolicy().hasHeightForWidth())
        self.epochValueLabel.setSizePolicy(sizePolicy)
        self.epochValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);")
        self.epochValueLabel.setObjectName("epochValueLabel")
        self.gridLayout.addWidget(self.epochValueLabel, 6, 1, 1, 1)
        self.nodeConnectButton = QtWidgets.QPushButton(self.groupBox)
        self.nodeConnectButton.setObjectName("nodeConnectButton")
        self.gridLayout.addWidget(self.nodeConnectButton, 1, 3, 1, 1)
        self.versionValueLabel = QtWidgets.QLabel(self.groupBox)
        self.versionValueLabel.setText("")
        self.versionValueLabel.setObjectName("versionValueLabel")
        self.gridLayout.addWidget(self.versionValueLabel, 3, 1, 1, 1)
        self.softwareLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.softwareLabel.setFont(font)
        self.softwareLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.softwareLabel.setObjectName("softwareLabel")
        self.gridLayout.addWidget(self.softwareLabel, 2, 0, 1, 1)
        self.softwareValueLabel = QtWidgets.QLabel(self.groupBox)
        self.softwareValueLabel.setText("")
        self.softwareValueLabel.setObjectName("softwareValueLabel")
        self.gridLayout.addWidget(self.softwareValueLabel, 2, 1, 1, 1)
        self.peerIDLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.peerIDLabel.setFont(font)
        self.peerIDLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.peerIDLabel.setObjectName("peerIDLabel")
        self.gridLayout.addWidget(self.peerIDLabel, 4, 0, 1, 1)
        self.nodeUrlLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nodeUrlLabel.setFont(font)
        self.nodeUrlLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nodeUrlLabel.setObjectName("nodeUrlLabel")
        self.gridLayout.addWidget(self.nodeUrlLabel, 1, 0, 1, 1)
        self.peerIDValueLabel = QtWidgets.QLabel(self.groupBox)
        self.peerIDValueLabel.setText("")
        self.peerIDValueLabel.setObjectName("peerIDValueLabel")
        self.gridLayout.addWidget(self.peerIDValueLabel, 4, 1, 1, 1)
        self.epochLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.epochLabel.setFont(font)
        self.epochLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.epochLabel.setObjectName("epochLabel")
        self.gridLayout.addWidget(self.epochLabel, 6, 0, 1, 1)
        self.nodeUrlsComboBox = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nodeUrlsComboBox.sizePolicy().hasHeightForWidth())
        self.nodeUrlsComboBox.setSizePolicy(sizePolicy)
        self.nodeUrlsComboBox.setToolTip("")
        self.nodeUrlsComboBox.setEditable(False)
        self.nodeUrlsComboBox.setMinimumContentsLength(0)
        self.nodeUrlsComboBox.setObjectName("nodeUrlsComboBox")
        self.gridLayout.addWidget(self.nodeUrlsComboBox, 1, 1, 1, 1)
        self.nodeConnectionStatusLabel = QtWidgets.QLabel(self.groupBox)
        self.nodeConnectionStatusLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.nodeConnectionStatusLabel.setText("")
        self.nodeConnectionStatusLabel.setPixmap(QtGui.QPixmap(":/icons/disconnected"))
        self.nodeConnectionStatusLabel.setScaledContents(True)
        self.nodeConnectionStatusLabel.setObjectName("nodeConnectionStatusLabel")
        self.gridLayout.addWidget(self.nodeConnectionStatusLabel, 1, 2, 1, 1)
        self.refreshNodeButton = QtWidgets.QPushButton(self.groupBox)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshNodeButton.setIcon(icon)
        self.refreshNodeButton.setObjectName("refreshNodeButton")
        self.gridLayout.addWidget(self.refreshNodeButton, 1, 4, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout_2.addWidget(self.groupBox, 0, QtCore.Qt.AlignTop)
        self.groupBox_2 = QtWidgets.QGroupBox(ConnectionWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.indexerUrlsComboBox = QtWidgets.QComboBox(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.indexerUrlsComboBox.sizePolicy().hasHeightForWidth())
        self.indexerUrlsComboBox.setSizePolicy(sizePolicy)
        self.indexerUrlsComboBox.setObjectName("indexerUrlsComboBox")
        self.gridLayout_2.addWidget(self.indexerUrlsComboBox, 0, 1, 1, 1)
        self.indexerUrlLabel = QtWidgets.QLabel(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.indexerUrlLabel.sizePolicy().hasHeightForWidth())
        self.indexerUrlLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.indexerUrlLabel.setFont(font)
        self.indexerUrlLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.indexerUrlLabel.setObjectName("indexerUrlLabel")
        self.gridLayout_2.addWidget(self.indexerUrlLabel, 0, 0, 1, 1)
        self.indexerConnectButton = QtWidgets.QPushButton(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.indexerConnectButton.sizePolicy().hasHeightForWidth())
        self.indexerConnectButton.setSizePolicy(sizePolicy)
        self.indexerConnectButton.setObjectName("indexerConnectButton")
        self.gridLayout_2.addWidget(self.indexerConnectButton, 0, 3, 1, 1)
        self.indexerBlockLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.indexerBlockLabel.setFont(font)
        self.indexerBlockLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.indexerBlockLabel.setObjectName("indexerBlockLabel")
        self.gridLayout_2.addWidget(self.indexerBlockLabel, 1, 0, 1, 1)
        self.indexerBlockValueLabel = QtWidgets.QLabel(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.indexerBlockValueLabel.sizePolicy().hasHeightForWidth())
        self.indexerBlockValueLabel.setSizePolicy(sizePolicy)
        self.indexerBlockValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);")
        self.indexerBlockValueLabel.setObjectName("indexerBlockValueLabel")
        self.gridLayout_2.addWidget(self.indexerBlockValueLabel, 1, 1, 1, 1)
        self.indexerConnectionStatusLabel = QtWidgets.QLabel(self.groupBox_2)
        self.indexerConnectionStatusLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.indexerConnectionStatusLabel.setText("")
        self.indexerConnectionStatusLabel.setPixmap(QtGui.QPixmap(":/icons/disconnected"))
        self.indexerConnectionStatusLabel.setScaledContents(True)
        self.indexerConnectionStatusLabel.setObjectName("indexerConnectionStatusLabel")
        self.gridLayout_2.addWidget(self.indexerConnectionStatusLabel, 0, 2, 1, 1)
        self.refreshIndexerButton = QtWidgets.QPushButton(self.groupBox_2)
        self.refreshIndexerButton.setIcon(icon)
        self.refreshIndexerButton.setObjectName("refreshIndexerButton")
        self.gridLayout_2.addWidget(self.refreshIndexerButton, 0, 4, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.verticalLayout_2.addWidget(self.groupBox_2)

        self.retranslateUi(ConnectionWidget)
        QtCore.QMetaObject.connectSlotsByName(ConnectionWidget)

    def retranslateUi(self, ConnectionWidget):
        #  _translate = QtCore.QCoreApplication.translate
        ConnectionWidget.setWindowTitle(_("Connection"))
        self.groupBox.setTitle(_("Node"))
        self.unsafeAPIExposedLabel.setText(_("Unsafe API exposed"))
        self.nodeBlockValueLabel.setText(_("?"))
        self.nodeBlockLabel.setText(_("Block"))
        self.versionLabel.setText(_("Version"))
        self.epochValueLabel.setText(_("?"))
        self.nodeConnectButton.setToolTip(_("Connection to node"))
        self.nodeConnectButton.setText(_("Connect"))
        self.softwareLabel.setText(_("Software"))
        self.peerIDLabel.setText(_("Peer ID"))
        self.nodeUrlLabel.setText(_("Url"))
        self.epochLabel.setText(_("Epoch"))
        self.refreshNodeButton.setToolTip(_("Refresh node infos from network"))
        self.refreshNodeButton.setText(_("Refresh"))
        self.groupBox_2.setTitle(_("Indexer"))
        self.indexerUrlLabel.setText(_("Url"))
        self.indexerConnectButton.setToolTip(_("Connection to the indexer"))
        self.indexerConnectButton.setText(_("Connect"))
        self.indexerBlockLabel.setText(_("Block"))
        self.indexerBlockValueLabel.setText(_("?"))
        self.refreshIndexerButton.setToolTip(_("Refresh indexer infos from network"))
        self.refreshIndexerButton.setText(_("Refresh"))
import tikka.slots.pyqt.resources.icons.index_rc
